/* RecordFile.c parser/unblocker test program */ /* Created by Larry Cochrane Date: 7/01/04 */
/* Modified by Karl Cunningham 2/24/10.  */

/* Changes to support gcc compiler are:
 * 1. add #includes for stdlib.h and string.h
 * 2. Change printf type designation fro %u to %lu for unsigned longs.
 * 3. Change the return type of main() to int.
 * 4. argc is one larger than in VC++
 */

#include "stdlib.h"
#include "stdio.h"
#include "time.h"
#include "string.h"
#include "unistd.h"
#include "sdr2seife.h"	

#define max(a,b) (((a) > (b)) ? (a) : (b))
#define min(a,b) (((a) < (b)) ? (a) : (b))

/*
 * This macro calculates the number of the next sample within a block after a given time 
 *  (in milliseconds after midnight UTC), the time tick for the start of the 
 *  block (in millieconds after midnight UTC), and the sample rate (in samples 
 *  per second). This will compute the sample number for a given channel if the
 *  sample rate is given in samples per second per channel, or for the unblocked
 *  data if the sample rate is given for all channels.
 *
 * Note that the sample number returned can be negative if the tick time is after the 
 *  desired sample time.
 */
#define sampleFromTime(time,tick,msPerSample) ((int)((((long)time) - ((long)tick)) / (msPerSample)))

/*
 * This macro calcaultes the time of a given sample in a block, given the sample number,
 *  the time tick for the start of the block (in milliseconds after midnight UTC), 
 *  and the sample rate (in samples per secand).
 */
#define timeFromSample(sample,tick,msPerSample) (((ULONG)tick) + ((ULONG)sample) * (msPerSample))
	
/*
 * Global Variables
 */
HeaderBlock hdrBlock; // Header block of the record file
int msPerSample; // Milliseconds per sample at the file's sample rate
int flagLen; // Length of the flag block for this file in bytes

int main(int argc, char *argv[])	
{
	FILE *fp,*outfile = NULL;
	BYTE *dataBlk;
    BYTE *flagBlk;
	FileInfo *finfo;
	InfoBlock *infoBlk;
	short *tmpData;		// holds 16 bit data
    /* 16 bit user data, which is unpacked into this array. The array is a 2-dimensional dynamic array. */
    short *userData[MAX_CHANNELS];
	int cnt, status, index, chan, samplesOutput = 0,gap,startingSample,lastSample,maxSample;
    int samplesThisBlock,chanSamplesThisBlock,startMinutes,dataSize,startSeconds,lastTickPrevBlock = 0;
    int beforeStart,pass;
    ULONG lastSampleTime,samplesToSend;
	ULONG startingTime;
    char tmpStr[40],tmpStr1[40];
    ClParms cliParameters;
		
    if(parseCommandLine(argc,argv,&cliParameters)) {
        usage();
        return 1;
    }

	/* This information should be used if the program is compiled using
	   a different compiler other then VC+++ 6.0 */
	//printf("Structure Sizes: HeaderBlock = %d FileInfo = %d InfoBlock = %d\n", 
	//	sizeof(HeaderBlock), sizeof(FileInfo), sizeof(InfoBlock));
		
	/* Open the record file */
	if(!(fp = fopen(argv[optind], "rb")))  {
		printf("Can't open input file\n");
		return 1;
	}
	
	/* Read in the header block */
	if((fread(&hdrBlock, 1, sizeof(hdrBlock), fp)) != sizeof(hdrBlock))  {
		printf("Header Read Error\n");
		fclose(fp);
		return 1;
	}
		
	/* Test the header version. The version number of the file will change if any 
		of the structures in the header block are changed */
	if(!testHeader(&hdrBlock))  {
		printf("Bad Header Check\n");
		fclose(fp);
		return 1;
	}
    
    // Check that the channel number given by the user is valid.
    if(cliParameters.chan > hdrBlock.numChannels) {
        fprintf(stderr,"Channel number invalid or not specified. Must be 1 through %d for this file.\n",hdrBlock.numChannels);
        return 1;
    }
	
	/* 
     * Display the information in the header, and save the number of ms (milliseconds) per sample.
     */
	displayHeader(argv[optind]);
    msPerSample = 1000 / hdrBlock.sampleRate;

	/*  Each data block has an array of flags to tell the unpacker if the sample is 
	    stored as a 8 bit character (+-127) or a short (+- 32K). This calculates
	    the the size of the array. WinSDR saves this array right after the InfoBlock
		and before the compressed data. It's the number of samples per second (all channels)
        times 60 seconds divided by 8 bits per byte, plus one in case the division had a remainder.
    */
	flagLen = ((hdrBlock.numSamples * 60) / 8) + 1; // In bytes

    if (allocatemem(&infoBlk,&flagBlk,&dataBlk,&tmpData,&userData)) {
        printf("Error allocating memory\n");
        return 1;
    }

    // Send some information about what we're doing
    printf("User Selections\n");
    printf("   Channel: %d\n",cliParameters.chan+1);
    if(cliParameters.startTime) {
        timeTick2String(cliParameters.startTime,tmpStr);
        printf("   Start Time: %s\n",tmpStr);
    }
    if(cliParameters.endTime) {
        timeTick2String(cliParameters.endTime,tmpStr);
        printf("   End Time:   %s\n",tmpStr);
    }
    if(*(cliParameters.heading)) {
        printf("   File Heading: \"%s\"\n",cliParameters.heading);
    }
    printf("\n");


	/* 
     * Now go through the file twice. Once to tally up the total number of samples we'll
     *  be sending to the output, and once for real sending the to the output file.
     */
    for(pass = 0; pass < 2; pass++) {
        status = 0;
        index = 0;
        cnt = 0;
        lastTickPrevBlock = 0;
        beforeStart = TRUE;
        while(!status)  {
            /*
             * If we have read in a block previously, save the time of the last sample for that block,
             *  in ms. This will be used to detect gaps.
             */
            if(index) 
                lastTickPrevBlock = lastSampleTime;

            // Use the correct fileInfo structure as we read blocks.
            finfo = &hdrBlock.fileInfo[index++];


            // readBlock returns -1 for end of file, +1 for error, and zero for successful read.
            if (status = readBlock(fp,finfo,infoBlk,flagBlk,dataBlk)) {
                if(status < 0) {
                //    printf("End of File Reached in Block %d\n",index);
                } else {
                    printf("Error Reading Block in Block %d\n",index);
                }
                continue;
            }

            /* Display the information in the block */
            // displayInfoBlock(index, infoBlk);

            /* 
             * Determine the size of the data block and decompress 16 bit data block into tmpData.
             *  Then save the lastSampleTime for this block. It is in ms (ms since 00Z)
             */
            dataSize = infoBlk->blockSize - flagLen - sizeof(InfoBlock);
            samplesThisBlock = decompressBuffer(tmpData, flagBlk, dataBlk, dataSize);
            cnt += samplesThisBlock;
            chanSamplesThisBlock = samplesThisBlock / hdrBlock.numChannels;
            lastSampleTime = timeFromSample(chanSamplesThisBlock-1,infoBlk->startTimeTick,msPerSample);

            /* Unblock the data into the user data block */
            unblockData(tmpData,userData,chanSamplesThisBlock);

            /*
             * Determine the gap between the last sample of the previous block and the first sample of this block. 
             *  There is always a gap of at least one sample, so what we calculate here is how much greater than 
             *  one sample the gap is.
             */
            if(lastTickPrevBlock) 
                gap = infoBlk->startTimeTick - lastTickPrevBlock - msPerSample;  // in milliseconds.

            /*
             * If we haven't found the startTime yet, look for it to possibly be available in the current block.
             *  If the time of the last sample in this block is greater than the start time, then it definitely 
             *  exists in this block. In that case, find the first sample equal to or after the startTime, open the 
             *  output file, send the header and the first line of data, and set beforeStart = FALSE.
             */
            if(beforeStart) {
                /*
                 * If the endtTime was specified by the user and the starting time for this block is after the endTime,
                 *  we can stop looking for samples and end, because there are no samples to mee the criteria.
                 */
                if((cliParameters.endTime) && cliParameters.endTime < infoBlk->startTimeTick) {
                    status = -1;
                    continue;
                }
                    
                if(lastSampleTime >= cliParameters.startTime) {
                    //printf("Found sample in block %d, startTimeTick: %lu lastSampleTime: %lu  startTime: %lu\n",
                    //      index,infoBlk->startTimeTick,lastSampleTime,cliParameters.startTime);
                    // printf("Finished unblocking data\n");

                    /*
                     * Get the sample number from the time, then the time from that sample number. The time returned might
                     *  be different if there wasn't a sample to exactly match the startTime.
                     */
                    // printf("Starting Sample from Time: %d\n",sampleFromTime(cliParameters.startTime,infoBlk->startTimeTick,msPerSample));
                    startingSample = max(0,sampleFromTime(cliParameters.startTime,infoBlk->startTimeTick,msPerSample));
                    startingTime = timeFromSample(startingSample,infoBlk->startTimeTick,msPerSample);

                    // Open the output file and write the header and first data lines
                    // printf("Opening output file\n");
                    if(pass == 1) {
                        if(!(outfile = fopen(argv[argc - 1],"wb"))) {
                            printf("Error opening output file for writing\n");
                            return 1;
                        }
                        fprintf(outfile,"%s\r\n",cliParameters.heading);
                    }
                    startMinutes = startingTime / 60000;
                    startSeconds = startingTime % 60000;
                    // printf("outputting the first record for channel %d, Found in block %d sample number %d\n",cliParameters.chan,index,startingSample);
                    if(pass == 1) {
                        sendHeader(outfile,samplesToSend,startMinutes,startSeconds);
                    }

                    beforeStart = FALSE;
                }
            }
                    
            /*
             * If we have found the first sample, process this block.
             */
            if(!beforeStart) {
                //printf("Feeding to output for block number %d\n",index);
                
                /*
                 * The gap between the end of the previous block and the start of this one should be the time of one 
                 *  sample. If it is two samples or more, output a warning to the user.
                 */
                if(lastTickPrevBlock && (gap >= msPerSample)) {
                    timeTick2String(gap,tmpStr);
                    timeTick2String(lastTickPrevBlock,tmpStr1);
                    if(pass == 1) {
                        printf("WARNING -- Gap of %s (%d samples) found in data at %s, between blocks %d and %d\n",
                            tmpStr,gap / msPerSample,tmpStr1,index-1,index);
                    }
                }
                /*
                 * Compute the last sample to send. Note that this is the number of the last sample, and is one
                 *  less than the number of samples in this block up to and including it. If greater than or equal to 
                 *  chanSamplesThisBlock, send the whole block. If not, the last sample is contained in this block,
                 *  so just enough samples, and set the status to -1 to indicate this is the last block to process.
                 */
                lastSample = max(0,sampleFromTime(cliParameters.endTime,infoBlk->startTimeTick,msPerSample));
                if(!cliParameters.endTime || (lastSample >= chanSamplesThisBlock)) {
                    maxSample = chanSamplesThisBlock - 1;
                } else {
                    //printf("desired last sample: %d within this block: %d this block contains %d samples\n",lastSample,index,chanSamplesThisBlock);
                    maxSample = lastSample;
                    status = -1;
                }
                /*
                 * Don't bother sending any data if the starting sample is after the max sample. This can happen if
                 *  the first sample sent above is actually the last sample in the current block.
                 */
                if(startingSample <= maxSample) {
                    if(pass > 0) {
                        feedToOutput(outfile,userData,startingSample,maxSample,cliParameters.chan);
                    } else {
                        samplesOutput += maxSample - startingSample + 1;
                    }
                }

                // Set startingSample to zero so we output starting at the beginning of subsequent blocks.
                startingSample = 0;
            }
        }
        /*
         * If this is the [end of the] first pass, save the samplesToSend and reset the file pointer to just after the header.
         */
        if(pass == 0) {
            samplesToSend = samplesOutput;
            printf("Resetting fseek\n");
            fseek(fp,sizeof(HeaderBlock),SEEK_SET);
        }
    }

    printf("\nFinished\n");
    printf("   Blocks read: %d   \n",index);
    printf("   Samples read: %d   \n",cnt);
    printf("   Samples per channel: %d  \n",cnt / hdrBlock.numChannels);
    printf("   Samples sent to output file: %lu\n",samplesToSend);
    printf("\n");

	/* Done reading the file. Now do some clean up. */
	fclose(fp);
    if(outfile != NULL) 
        fclose(outfile);
	free(dataBlk);
    free(infoBlk);
    free(flagBlk);
    free(tmpData);
    for (chan = 0; chan < hdrBlock.numChannels; chan++) {
        free(userData[chan]);
    }

}

int feedToOutput(FILE *outfile,short *userData[],int startingSample,int maxSample,int chan)
{
    /*
     * Sends samples from the selected channel to the output file, formatted according to the SEIFE file
     *  format. Starts at the sample number given.
     */
    int sample,cnt = 0;

    // printf("Sending data to output starting at sample %d and ending at sample %d\n",startingSample,maxSample);
    for(sample = startingSample;sample <= maxSample;sample++) {
        fprintf(outfile,"%d\r\n",userData[chan-1][sample]);
        cnt++;
    }
    return cnt;
}


void sendHeader(FILE *outfile,ULONG samplesToSend,int startMinutes,int startSeconds)
{
    /*
     * Writes the header information to the output file. This is done after we know there are 
     *  samples to send to the file. We generate a string for each parameter and then output them
     *  to the file, rather than do it in one shot because printf cannot left justify and pad with 
     *  leading zeros at the same time.
     */
    char strSmp[20];
    char strTPS[20];
    char strMin[20];
    char strSec[20];

    sprintf(strSmp,"%lu",samplesToSend);
    sprintf(strTPS,"%.3f",msPerSample / 1000.0);
    sprintf(strMin,"%d.",startMinutes);
    sprintf(strSec,"%d.%03d",startSeconds / 1000,startSeconds % 1000);
    fprintf(outfile,"%-10s%-20s%-10s%-10s%-10s\r\n",strSmp,"(1f10.0)",strTPS,strMin,strSec);
}

void usage(void)
{

}

int parseCommandLine(int argc,char **argv,ClParms *cliParameters)
{
    /*
     * Uses getopt to parse the command line, and decodes the response
     *  Options are -c<channel>  -r<samplerate  -t<starttime>  -n<numsamples>
     *    numsamples is at the output samplerate.
     */
    int optret;
    char *chan,*rate,*time,*samples;

    // Set all the pointers to null/zero so we can tell if we got them.
    cliParameters->chan = 0;
    cliParameters->startTime = 0LL;
    cliParameters->endTime = 0LL;
    cliParameters->heading = "";

    opterr =0;
    while((optret = getopt(argc,argv,"c:e:s:h:")) != -1) {
        switch (optret)
        {
            case 'c':
                sscanf(optarg,"%d",&(cliParameters->chan)); // Channel number
                //printf("Got channel: %d\n",cliParameters->chan);
                break;
            case 's':
                //printf("Start Time string: %s\n",optarg);
                if((cliParameters->startTime = convertTime(optarg)) < 0) {
                    return 1;
                }
                //printf("Got startTime string: %s time value: %lu\n",optarg,cliParameters->startTime);
                break;
            case 'e':
                //printf("End Time string: %s\n",optarg);
                if((cliParameters->endTime = convertTime(optarg)) < 0) {
                    return 1;
                }
                //printf("Got endTime string: %s time value: %lu\n",optarg,cliParameters->endTime);
                break;
            case 'h':
                cliParameters->heading = optarg;
                //printf("Got heading: %s\n",cliParameters->heading);
                break;
            case '?':
                // Invalid option
                if(isprint(optopt))
                    fprintf(stderr,"Invalid Option: '-%c'.\n",optopt);
                else
                    fprintf(stderr,"Invalid Option Character: '\\x%x'.\n",optopt);
                return 1;
            default:
                fprintf(stderr,"In parseCommandLine Unhandled optret: %d\n",optret);
                return 1;
        }
    }
    // Now check for the required switches.
    if((cliParameters->chan <= 0) || (cliParameters->chan > MAX_CHANNELS)) {
        fprintf(stderr,"Channel number invalid or not specified\n");
        return 1;
    }

    // End time must be after start time.
    if(cliParameters->endTime && (cliParameters->endTime <= cliParameters->startTime)) {
        fprintf(stderr,"End Time must be after Start Time\n");
        return 1;
    }
    // The arguments must be at least two.
    if((argc - optind) < 1) {
        fprintf(stderr,"ERROR -- Missing Input Filename \n");
        return 1;
    } 
    // The arguments must be at least two.
    if((argc - optind) < 2) {
        fprintf(stderr,"ERROR -- Missing Output Filename\n");
        return 1;
    }

    //printf("Channel Number: %d\n",cliParameters->chan);
    //printf("Start Time: %lu\n",cliParameters->startTime);
    //printf("End Time: %lu\n",cliParameters->endTime);
    //printf("Heading: %s\n",cliParameters->heading);
    return 0;
}

long long convertTime(char *dateStr)
{
    /*
     * Converts a time of the format HH:MM:SS.ddd or MM:SS.ddd to a long long unix time in milliseconds.
     *  returns the time.  Ignores the day.
     */
    int hour = 0,min = 0,cnt;
    float sec = 0.0;
    char saved[256][256];
    const char colon[] = ":"; // The delimeter
    char *mytok;
    char mydate[250];

    strcpy(mydate,dateStr);
    mytok = strtok(mydate,colon);
    cnt = 0;
    while(mytok != NULL) {
        strcpy(saved[cnt++],mytok);
        mytok = strtok(NULL,colon);
    }

    switch (cnt) 
    {
        case 0:
            // Invalid
            printf("Invalid Time String\n");
            return -1;
            break;
        case 1:
            // One value. Assuming seconds
            if(!sscanf(saved[0],"%f",&sec)) {
                printf("Invalid Time String -- Could not get seconds\n");
                return -1;
            }
            break;
        case 2:
            // Decode minutes & seconds
            if(!sscanf(saved[0],"%d",&min)) {
                printf("Invalid Time String -- Could not get minutes\n");
                return -1;
            }
            if(!sscanf(saved[1],"%f",&sec)) {
                printf("Invalid Time String -- Could not get seconds\n");
                return -1;
            }
            break;
        case 3:
            // Decode hours, minutes, & seconds
            if(!sscanf(saved[0],"%d",&hour)) {
                printf("Invalid Time String -- Could not get hours\n");
                return -1;
            }
            if(!sscanf(saved[1],"%d",&min)) {
                printf("Invalid Time String -- Could not get minutes\n");
                return -1;
            }
            if(!sscanf(saved[2],"%f",&sec)) {
                printf("Invalid Time String -- Could not get seconds\n");
                return -1;
            }
            break;
        default:
            printf("Invalid Time String\n");
            return -1;
            break;
    }
    min += sec / 60;
    sec -= (int)sec / 60 * 60;
    hour += min / 60;
    min %= 60;
    return (long long)((hour * 3600 + min * 60 + sec) * 1000);
}

int allocatemem(InfoBlock **infoBlk,BYTE **flagBlk,BYTE **dataBlk,short **tmpData,short *(userData[][MAX_CHANNELS]))
{
    int chan;

    /* Create a buffer to hold the fileINfo header for each block */
    if(!(*infoBlk = (InfoBlock *)malloc(sizeof(InfoBlock)))) {
        printf("Out of memory allocating for the info block buffer!\n");
        return 1;
	}	

    /* Create a buffer to hold the flag data */
    if(!(*flagBlk = (BYTE *)malloc(flagLen))) {
        printf("Out of Memory allocating for the flag array block!\n");
        return 1;
    }
	
	/* Create a buffer large enough to hold one block of channel data, assuming the worst case of all 16-bit data. */
    if(!(*dataBlk = (BYTE *)malloc((sizeof(short) * 60 * hdrBlock.numSamples))))  {
        printf("Out of memory allocating for the data block!\n");
        return 1;
	}	
	
	/* Create a buffer to old the decompressed data.  HdrBlock.numSamples is in 
	   samples per second, so the 60 is to make the buffer large enough to hold
	   one minute's worth of data. */
    if(!(*tmpData = (short *)malloc(sizeof(short) * 60 * hdrBlock.numSamples)))  {
        printf("Out of memory allocating the tmpData array!\n");
        return 1;
	}

    /* Create a buffer to hold userData, which is one block of data divided into channels */
    for (chan = 0; chan < hdrBlock.numChannels; chan++) {
        if(!((*userData)[chan] = (short *)malloc(sizeof(short) * MAX_USER_LEN)))  {
            printf("Out of Memory allocating the userData array!\n");
            return 1;
        }
    }


    return 0;
}


void sumData(short *userData[MAX_CHANNELS],double sums[],int samplesToCheck) 
{
    /* Adds the data for each channel up so we can compute an average 
     *  Uses data in the userData array 
     */
    int chan, sample;

    for(chan = 0; chan < hdrBlock.numChannels; chan++)
        for (sample = 0; sample < samplesToCheck; sample++)
            sums[chan] += userData[chan][sample];
}

int readBlock(FILE *fp,FileInfo *finfo,InfoBlock *infoBlk,BYTE *flagBlk,BYTE *dataBlk)
{
    /* Reads a block of channel data into the buffers. First reads the 8/16-bit flag array and the 
     *  block header info into their buffers. Returns 0 on success, -1 for end of file, +1 for error.
     *  finfo contains information about this block, from the FileInfo structure at the start of the file.
     *  infoBlk is the info structure at the beginning of this block.
     *  flagBlk is the flag array for this block
     *  dataBlk is the data field for this block.
     */
    int status,flagsize,pointsperblock,size;
    ULONG fpos;


    // we should be at the correct file position
    fpos = ftell(fp);
    //if(fpos != finfo->filePosition) {
     //   printf("Position Delta: %ld     ",fpos - finfo->filePosition);
    //}
    
    /*if(fpos != finfo->filePosition) {
        printf("ERROR -- Incorrect file position! Should be here: %lu but instead are here: %lu delta: %lu\n", 
            finfo->filePosition, fpos,finfo->filePosition - fpos);
        return 1;
    }
    */
    
    /* First read in the InfoBlock part of the data block */
    if((status = fread(infoBlk, 1, sizeof(InfoBlock), fp)) != sizeof(InfoBlock))  {
        //printf("FINISHED -- End of File Reading infoBlk\n");
        return -1;
    }

    /* Check for a good block ID */
    if(infoBlk->goodID != GOOD_BLK_ID)  {
        printf("ERROR -- Bad Header Check ID\n");
        return 1;
    }

    /* Now read in the flag array */
    if((status = fread(flagBlk, 1, flagLen, fp)) != flagLen) {
        printf("ERROR -- End of file reading the flag array\n");
        return 1;
    }

    /* Calculate the data part of the block */
    size = infoBlk->blockSize - flagLen - sizeof(InfoBlock);
    
    /* Read in the rest of the block of data*/
    if((status = fread(dataBlk, 1, size, fp)) != size)  {
        printf("Block Read Error\n");
        return 1;
    }

    // Return 1 to indicate successful read of the block of data.
    return 0;
}

/* Decompress the data into the short data *data.  */
int decompressBuffer(short *data, BYTE *flagBlk, BYTE *dataBlk, int dataSize)
{
    /* Extracts data from the block read from the file. The file data is either 8- or 16-bit
     * for each reading. The flagBlk array contains a flag for each data point telling whether it
     * is 8- or 16-bit data.
     */
	int mask = 1, bitCount = 0;
	char *ptr;
    short *outptr = data;
    int count = 0;

	ptr = (char *)dataBlk;
	
	/* Now decompress a block's worth of data */
	while(dataSize > 0)  {
		if(*flagBlk & mask)  {				// check for a short sample
			*outptr++ = *((short *)ptr);	// save the sample 
			ptr += 2;					// point to the next sample
            dataSize -= 2;
		} else{							// not a short must be char
			*outptr++ = *ptr++;			// save the sample
            dataSize--;
        }
		if(++bitCount >= 8)  {			// next bit
			bitCount = 0;
			mask = 1;
			++flagBlk;
		}
		else
			mask <<= 1;
        count++;
	}
    return count;
}
		
/* Unblock the 16 bit data to the user data array */
void unblockData(short *inData,short *userData[MAX_CHANNELS],int samplesThisBlock)
{
    /* This function separates the blocked data into channel data. The blocked data
     *  is sequential by channel.
     */
	int chan, sample;
    short *inptr = inData;
	
    for (sample = 0; sample < samplesThisBlock; sample++) {
        for(chan = 0; chan < hdrBlock.numChannels; chan++)
            userData[chan][sample] = *inptr++;
	}	
}

/* Display the header block information */
void displayHeader(char *infilename)
{
	char tmStr[40];
    
    printf("\n");
    printf("Header information from input file:   %s\n",infilename);
	
	printf("   Number of Channels: %d\n", hdrBlock.numChannels);
	printf("   Sample Rate: %d\n", hdrBlock.sampleRate);
	printf("   Total Samples per Second: %d\n", hdrBlock.numSamples);
	printf("   Number of Data Blocks: %d\n", hdrBlock.numBlocks);
	printf("   Last Block Size: %d bytes   Last File Offset %lu bytes\n", hdrBlock.lastBlockSize, hdrBlock.lastBlockOffset);
	makeTimeStr(hdrBlock.startTime, tmStr);
	printf("   First Block Start Time: %s\n", tmStr);
	makeTimeStr(hdrBlock.lastTime, tmStr);
	printf("   Last Block Start Time: %s\n\n", tmStr);
}

/* Display the data in the InfoBlock structure */
void displayInfoBlock(int blockno, InfoBlock *blk)
{
	char *lckPtr, tmStr[40], alarms[80], tmp[16];
	int c;
	
	if(blk->flags & F_ISLOCKED)
		lckPtr = "(L)";
	else if(blk->flags & F_WASLOCKED)
		lckPtr = "(?)";
	else
		lckPtr = "(N)";
	makeDateTimeStr(tmStr, blk->startTime, blk->startTimeTick);
	alarms[0] = 0;
	for(c = 0; c < hdrBlock.numChannels; c++)  {
		if(blk->alarmFlags[c])  {
			sprintf(tmp, "%d ", c+1);
			strcat(alarms, tmp);
		}
	}
	printf("Data Blk No: %d %s %s Size: %lu Alarm Chans: %s\n", blockno, tmStr, lckPtr, blk->blockSize, alarms);
}

void timeTick2String(ULONG tick,char *output)
{
    /*
     * Converts a timeTick value (milliseconds from midnight UTC) to a string
     *  of the form hh:mm:ss.ddd.
     */
    int hh,mm,ss,ms;

    hh = tick / 3600000;
    mm = (tick % 3600000) / 60000;
    ss = (tick % 60000) / 1000;
    ms = tick % 1000;
    sprintf(output,"%d:%02d:%02d.%03d",hh,mm,ss,ms);
}

/* Make a time UTC based string */
void makeTimeStr(long ltime, char *str)
{
	struct tm *nt;
	
	nt = gmtime(&ltime);
	sprintf(str, "%02d/%02d/%02d %02d:%02d:%02d", nt->tm_mon+1, nt->tm_mday, 
		nt->tm_year % 100, nt->tm_hour, nt->tm_min, nt->tm_sec);
}

/* Make a time UTC based string based on the InfoBlck time information */
void makeDateTimeStr(char *str, long ltime, long tick)
{
	struct tm *nt;
	long tmp;
	int hour, min, sec, millsec;
	
	millsec = (int)(tick % 1000L); tick /= 1000L;
	hour = (int)(tick / 3600L); tmp = tick % 3600L;
	min = (int)(tmp / 60L);
	sec = (int)(tmp % 60L);
	nt = gmtime(&ltime);
	sprintf(str, "%02d/%02d/%04d %02d:%02d:%02d.%03d", nt->tm_mon+1, nt->tm_mday, 
		nt->tm_year+1900, nt->tm_hour, min, sec, millsec);
}

/* Test the header file verison */
int testHeader(HeaderBlock *blk)
{
	UINT ver = blk->fileVersionFlags & 0xFF;
	if(ver != HDR_VERSION)
		return(FALSE);
	return(TRUE);
}




