/* Header file for the RecordFile.c program. */
/* Created by Larry Cochrane Data: 7/01/04 */

#include <stdio.h>
#include <time.h>	
	
#define ULONG			unsigned long		// 32 bits
#define UINT			unsigned int		// 32 bits
#define WORD			unsigned short		// 16 bits
#define BYTE			unsigned char		// 8 bits
#define TRUE			1
#define FALSE			0

#define MAX_USER_LEN	(200*60)	// Maximum user data buffer size for each channel 

#define MAX_CHANNELS	8			// Maximum number of channels
#define MAX_FILE_INFO	2000		// Number of FileInfo blocks
#define HDR_VERSION		0x01		// Version ID of the record file
#define GOOD_BLK_ID		0xa55a		// Each data block has this at the beginning of block

// The flags in the InfoBlock below
#define F_ISLOCKED		0x01		// Indicates that the A/D board was locked to a reference source
#define F_WASLOCKED		0x02		// Indicates that the A/D board was locked at some point but not
									
/* This structure is contained in the main header block. It is used
   to find the start location of data based on the start time of the block.
   WinSDR updates the header and the information in this block every minute.
   This is the interval that the program saves the data to disk. */
typedef struct  {
	ULONG startTime, 				// Start time (UTC) of the data block
		filePosition;				// The start position in the record file 
	int blockSize, 					// The size of the first block
		julian;						// The julian day of the block
} FileInfo;

// The flags in the HeaderBlock fileVersionFlags below 

/* This is the main header block at the beginning of the file */
typedef struct  {
	int fileVersionFlags,		// Holds some unused flags and the file version number
		sampleRate,				// Sample rate of each channel in samples per second per channel
		numSamples,				// Number of samples per second for all channels.
		numChannels,			// Number of channels
		numBlocks,				// Number of data blocks (1 minute each)
		lastBlockSize;			// The size of the last data block
	ULONG startTime, lastTime,	// Start and End times of the file
		 lastBlockOffset;		// The offset to the last data block
	FileInfo fileInfo[MAX_FILE_INFO];	// Holds information for each block
} HeaderBlock;
	
/* Every minute WinSDR saves the data from the A/D baord to disk. This is the structure at the beginning
   of the data block */
typedef struct  {
	WORD goodID,			// Set to  GOOD_BLK_ID
		 flags;				// See F_ flags above
	BYTE alarmFlags[MAX_CHANNELS]; // If TRUE WinSDR was in the alarm state for the channel
	ULONG startTime,		// Start time (UTC) of the block. Resolution down to the second
		 startTimeTick,		// Number of millisconds past midnight UTC of the block
		 blockSize;			// The data block size
} InfoBlock;

typedef struct {
    int chan; // Channel number (1 .. MAX_CHANNELS)
    ULONG startTime; // time for first sample in milliseconds from 0 UTC
    ULONG endTime; // time for last sample in milliseconds from 0 UTC
    char *heading; // header string to be sent to the output.
} ClParms;

/* test program functions */
void saveUserData();
int readBlock(FILE *fp,FileInfo *,InfoBlock *,BYTE *,BYTE *);
void displayHeader(char *);
void displayInfoBlock(int, InfoBlock *);
int decompressBuffer(short *, BYTE *, BYTE *, int);
void unblockData(short *,short *[MAX_CHANNELS],int);
void sumData(short *[MAX_CHANNELS],double *,int);
void makeTimeStr(long, char *);
void makeDateTimeStr(char *, long, long);
int testHeader(HeaderBlock *);
int allocatemem(InfoBlock **,BYTE **,BYTE **,short **,short *(userData[][MAX_CHANNELS]));
void usage(void);
long long convertTime(char *dateStr);
int feedToOutput(FILE *,short *[],int ,int, int);
void sendHeader(FILE *,ULONG ,int ,int );
void timeTick2String(ULONG tick,char *output);
